# coding: utf-8
########################################################################
#
# Linear stability analysis for 3D crack propagation
# in mixed-mode I+III accounting for T-stress
#
########################################################################
#
# Authors: Mathias Lebihain: Formal analysis, Software
#          Jean-Baptiste Leblond: Formal analysis

##Scientific packages
import numpy as np
import scipy.ndimage as ndi
import scipy.optimize as optim
import scipy.interpolate as interpolate
#Graphical packages
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.cm as cm
import matplotlib.ticker as ticker
from matplotlib.gridspec import GridSpec
from matplotlib.colors import Normalize, LogNorm
from mpl_toolkits.axes_grid1 import make_axes_locatable
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.collections import LineCollection
##Files managements packages
import os

#LateX font
from matplotlib import rc
rc('text', usetex=True)
#Folder creation
if not(os.path.isdir('data')) :
    os.mkdir('data')

if not(os.path.isdir('figures')) :
    os.mkdir('figures')

#Define colorblind-safe colors for the article
#Palettes extracted from https://colorbrewer2.org/
cm_darkblue = (37./255., 52./255., 148./255.)
cm_blue = (44./255., 127./255., 184./255.)
cm_greenblue = (127./255., 205./255., 187./255.)
cm_black = (0., 0., 0.)
cm_lightbrown = (254./255., 217./255., 142./255.)
cm_orange = (254./255., 153./255., 41./255.)
cm_brown = (204./255., 76./255., 2./255.)

########################################################################
## Parameters
########################################################################

########################################################################
##LSA functionals

##Terms in \rho^2
def F(rho, xi, nu):
    y = ((2.+nu)*xi - 3.*(2.-nu) + 4.*np.sqrt(2)*(1.-2.*nu)/(xi+1.)**0.5)*rho**2 + (1.-nu)*(2.-nu)*xi + (1.-nu)*(2.-3.*nu)
    return y

def f_xx(rho, xi, nu):
    y = -2.*np.sqrt(2)/(xi+1.)**1.5 * xi * (((2.+nu)*xi + 2.)*rho**2 +  (1.-nu)*((2.-nu)*xi + 2.))
    return y

def f_xz(rho, xi, nu):
    y = - 4.*np.sqrt(2)/(xi+1.)**1.5 * ((-2.*xi**2 - (1.-nu)*xi +1.)*rho**2 + (1.-nu)*((1.-nu)*xi + 1.))
    return y

def f_zz(rho, xi, nu):
    y = -2.*np.sqrt(2)/(xi+1.)**1.5 * ((4.*xi + 4-nu)*rho**2 + nu*(1.-nu))
    return y

def LSA_zero(rho, xi, sgn, q_xx, q_xz, q_zz, nu):
    y = F(rho, xi, nu) + f_xx(rho, xi, nu) * q_xx + 1j*sgn*f_xz(rho, xi, nu) * q_xz + f_zz(rho, xi, nu) * q_zz
    return y

########################################################################
#Find xi

#Define the polynomial of Eq.()
def u_poly(rho, sgn, q_xx, q_xz, q_zz, nu):
    if (q_xx == q_xz == q_zz == 0.):
        coeffs_poly = np.zeros(4, dtype='complex')
        coeffs_poly[0] = -4 * np.sqrt(2)* (1.-2.*nu) * rho**2
        coeffs_poly[1] = 2. * (4.-nu) * rho**2 + 2. * nu * (1.-nu)
        coeffs_poly[2] = 0.
        coeffs_poly[3] = -(2.+nu) * rho**2 - (1.-nu) * (2.-nu)
    else:
        coeffs_poly = np.zeros(6, dtype='complex')
        coeffs_poly[0] = 2*np.sqrt(2)*nu * (rho**2 - (1.-nu)) * (q_xx - 1j*sgn*2.*q_xz - q_zz)
        coeffs_poly[1] = 0.
        coeffs_poly[2] = 4*np.sqrt(2)* ((-1.+2.*nu - (1.+nu)*q_xx +1j*sgn*(3.+nu)*q_xz + 2.*q_zz)*rho**2 + (1.-nu)**2*(-q_xx + 1j*sgn*q_xz))
        coeffs_poly[3] = 2.*(4.-nu)*rho**2 + 2.*nu*(1.-nu)
        coeffs_poly[4] = 2*np.sqrt(2) * (((2.+nu)*q_xx - 1j*sgn*4.*q_xz)*rho**2 + (1.-nu)*(2.-nu)*q_xx)
        coeffs_poly[5] = -(2.+nu)*rho**2 - (1.-nu)*(2.-nu)
    
    return coeffs_poly

def find_xi(rho, sgn, q_xx, q_xz, q_zz, nu, sort):
    #Find roots of the polynomial in zeta = 1/sqrt(xi+1)
    zeta = np.roots(u_poly(rho, sgn, q_xx, q_xz, q_zz, nu)).astype('complex')
    #Restriction on zeta
    #Note: This ensures Re(xi)> -1 for the convergence of the integrals
    zeta = zeta[(zeta.real>0.) & (np.abs(zeta.imag)<zeta.real)]
    
    #Keep only the solutions Re(xi)> -1
    #This condition resumes to Re(zeta)>0 and |Im(zeta)|<Re(zeta)
    #Output NaN if no roots is valid
    xi = (1.-zeta**2)/zeta**2 if (zeta.size!=0) else np.array([np.nan])
    
    #Sort
    if (sort):
        xi_tuple = np.array([(round(x.real,6),round(x.imag,6)) for x in xi], dtype=[('Re', 'float'), ('Im', 'float')])
        xi = xi[np.argsort(xi_tuple, order=('Re', 'Im'))]
    
    return xi

def max_Re_xi(rho, sgn, q_xx, q_xz, q_zz, nu):
    xi = find_xi(rho, sgn, q_xx, q_xz, q_zz, nu, False)
    max_Re = np.nan if (np.all(np.isnan(xi.real))) else np.nanmax(xi.real)
    return max_Re

########################################################################
##Stability analysis of [Leblond et al., 2011]
########################################################################

#Poisson's ratio
nu = 0.3
#Mode mixity
rho = np.linspace(0., 2., num=2001, endpoint=True)
#Find xi
Re_xi = np.full((rho.size, 5), np.nan)
Im_xi = np.full((rho.size, 5), np.nan)
for j in range(rho.size):
    #Find xi
    xi = find_xi(rho[j], 1, 0., 0., 0., nu, True)
    if (xi.size==2):
        Re_xi[j,1:xi.size+1] =  np.copy(xi.real)
        Im_xi[j,1:xi.size+1] =  np.copy(xi.imag)
    elif ((xi.size==1) | (xi.size==3)):
        Re_xi[j,:xi.size] =  np.copy(xi.real)
        Im_xi[j,:xi.size] =  np.copy(xi.imag)
    else :
        print('Be careful with the roots sorting. N_roots={:d}>3'.format(xi.size))
        break

#Growth rate
growth_rate = Re_xi
#Drift
drift = np.arctan(-Im_xi)
drift[np.isclose(drift, 0.)] = 0.

##Find rho_cr 
#Filter full NaN columns
not_nan = np.logical_not(np.all(np.isnan(Re_xi), axis=1))
max_Re = np.full(rho.size, np.nan)
max_Re[not_nan] = np.nanmax(Re_xi[not_nan], axis=1)
#Check stability
is_unstable = max_Re >= 0.
k_max = np.argmax(is_unstable)
#Limits
rho_inf = rho[k_max-1]
rho_sup = rho[k_max]
Re_xi_inf = max_Re[k_max-1]
Re_xi_sup = max_Re[k_max]
#First guess from linear interpolation
rho_guess = rho_inf + (0.-Re_xi_inf)/(Re_xi_sup-Re_xi_inf)*(rho_sup-rho_inf)
#Function
g_rate = lambda rho : max_Re_xi(rho, 1, 0., 0., 0., nu)
#Find rho_cr so for which one wavelength is unstable
sol_rho = optim.root(g_rate, rho_guess, tol=1e-5, options=[('maxfev', 1000)])
rho_cr = sol_rho.x[0]

#Plot parameters
my_fontsize = 18
my_labelsize = 14
my_ms = 1
my_lw = 1.5
#Axis parameters
rho_min, rho_max = 0., rho.max()
rho_ticks = np.arange(0., rho.max()+0.1, step=0.5)
rho_labels = [r'${0:g}$'.format(rho) for rho in rho_ticks]
rate_min, rate_max = -1., 2.

#Figure
fig = plt.figure(figsize=(4,3))
gs = GridSpec(1, 1, figure=fig)
ax_01 = fig.add_subplot(gs[0,0])
ax_01.patch.set_facecolor('white')
#Plot
ax_01.axhspan(rate_min, 0., color='grey', linewidth=0., alpha=0.5, zorder=-1)
ax_01.axhline(0., color='grey', linewidth=0.5, linestyle=':', zorder=-1)
ax_01.axvline(rho_cr, color='firebrick', linewidth=1.5, linestyle='--')
for j in np.arange(5):
    ax_01.plot(rho, growth_rate[:,j], linewidth = my_lw, color='black', linestyle='-')

#Axes
for axis in ['top','bottom','left','right']:
	ax_01.spines[axis].set_linewidth(1.5)

ax_01.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.set_xticks(rho_ticks)
ax_01.set_xticklabels(rho_labels)
#Labels
ax_01.set_xlabel(r'Mode mixity ratio $K_\mathrm{III}^0/K_\mathrm{I}^0$', fontsize=my_fontsize)
ax_01.set_ylabel(r'Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$', fontsize=my_fontsize)
#Limits
ax_01.set_xlim(rho_min, rho_max)
ax_01.set_ylim(rate_min, rate_max)
##Fix position
#Interlines
L_x = 0.18
L_y = 0.2
#Graphic sizes
H_x = (0.95-L_x)
H_y = (0.95-L_y)
#Set position
ax_01.set_position([0.01+1*L_x, 0.01+0*H_y+1*L_y, H_x, H_y])
##Save
fig.savefig('figures/EJM_Leblond_2011.pdf', format='pdf')
plt.close()

#Data for Zenodo
export_header = r"Poisson's ratio $\nu$; Mode mixity ratio $K_\mathrm{III}^0/K_\mathrm{I}^0$; Sol. n°1 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°1 Drift angle $\alpha$; Sol. n°2 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°2 Drift angle $\alpha$; Sol. n°3 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°3 Drift angle $\alpha$"
export_table = np.column_stack([np.full(rho.size, nu), rho, growth_rate[:,0], drift[:,0], growth_rate[:,1], drift[:,1], growth_rate[:,2], drift[:,2]])
np.savetxt("data/data_Figure_2.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')

########################################################################
##Influence of T_{xx}
########################################################################
print("--- Part I: Influence of T_xx ---")

########################################################################
#Configuration stability for a fixed Poisson's ratio

#Poisson's ratio
nu = 0.3
print("I.a T_xx - Stability analysis for a Poisson's ratio nu={:.1f}".format(nu))

#Mode mixity
rho = np.linspace(0., 2., num=2001, endpoint=True)
#T-stress
L_pert_over_L_xx = np.array([-1., -0.5, 0., 0.5, 1., 1.5])
q_xx = np.sign(L_pert_over_L_xx)*np.sqrt(np.abs(L_pert_over_L_xx)/2./np.pi)
#Find xi
Re_xi = np.full((q_xx.size, rho.size, 5), np.nan)
Im_xi = np.full((q_xx.size, rho.size, 5), np.nan)
for i in range(q_xx.size):
    print("Computation: {:2d}%".format(int(100.*(i+1)/q_xx.size)), end='\r', flush=True)
    for j in range(rho.size):
        #Find xi
        xi = find_xi(rho[j], 1, q_xx[i], 0., 0., nu, True)
        if (xi.size==2):
            Re_xi[i,j,1:xi.size+1] =  np.copy(xi.real)
            Im_xi[i,j,1:xi.size+1] =  np.copy(xi.imag)
        elif ((xi.size==1) | (xi.size==3)):
            Re_xi[i,j,:xi.size] =  np.copy(xi.real)
            Im_xi[i,j,:xi.size] =  np.copy(xi.imag)
        else :
            print('Be careful with the roots sorting. N_roots={:d}>3'.format(xi.size))
            break

print("\n----- Completed -----")
#Growth rate
growth_rate = Re_xi
#Drift
drift = np.arctan(-Im_xi)
drift[np.isclose(drift, 0.)] = 0.

#Indices to plot
N_plot = L_pert_over_L_xx.size
indices_plot = np.arange(N_plot)
#Plot parameters
my_fontsize = 18
my_labelsize = 14
my_ms = 1
my_lw = [1.,1.,1.5,1.,1.,1.]
#Axis parameters
rho_min, rho_max = 0., rho.max()
rho_ticks = np.arange(0., rho.max()+0.1, step=0.5)
rho_labels = [r'${0:g}$'.format(rho) for rho in rho_ticks]
rate_min, rate_max = -1., 4.
drift_min, drift_max = -np.pi/2., np.pi/2.
alpha_ticks = [-np.pi/2.,-np.pi/4.,0.,np.pi/4.,np.pi/2.]
alpha_labels = [r'-$\frac{\pi}{2}$',r'-$\frac{\pi}{4}$', r'$0$', r'$\frac{\pi}{4}$', r'$\frac{\pi}{2}$']
#Create custom colormap for \ell_\mathrm{pert}/L_xx
cmap_list = [cm_blue, cm_greenblue, cm_black, cm_lightbrown, cm_orange, cm_brown]
#Colormap
cmap_plot = colors.LinearSegmentedColormap.from_list('Discrete cmap', cmap_list, L_pert_over_L_xx.size)
#Labels
cmap_labels = [r'${0:g}$'.format(L) for L in L_pert_over_L_xx]
N_labels = len(cmap_labels)
#Norm
norm_bins = np.arange(L_pert_over_L_xx.size)+0.5
norm_bins = np.insert(norm_bins, 0, np.min(norm_bins) - 1.0)
norm_diff = norm_bins[1:] - norm_bins[:-1]
cmap_ticks = norm_bins[:-1] + norm_diff / 2
cmap_norm = colors.BoundaryNorm(norm_bins, N_labels, clip=True)
cmap_fmt = ticker.FuncFormatter(lambda x, pos: cmap_labels[cmap_norm(x)])

#Figure
fig = plt.figure(figsize=(4,6))
gs = GridSpec(3, 1, figure=fig)

ax_01 = fig.add_subplot(gs[0,:])
ax_01.patch.set_facecolor('white')
#Plot
ax_01.axhspan(rate_min, 0., color='grey', linewidth=0., alpha=0.5, zorder=-1)
ax_01.axhline(0., color='grey', linewidth=0.5, linestyle=':', zorder=-1)
for i in range(N_plot):
    for j in np.arange(5):
        filter_straight = np.isclose(drift[i,:,j],0.)
        filter_drift = np.logical_not(filter_straight)
        ax_01.plot(rho[filter_straight], growth_rate[i,:,j][filter_straight], linewidth = my_lw[i], color=cmap_list[i], linestyle='-')
        ax_01.plot(rho[filter_drift], growth_rate[i,:,j][filter_drift], linewidth = my_lw[i], color=cmap_list[i], linestyle='--')

#Axes
for axis in ['top','bottom','left','right']:
	ax_01.spines[axis].set_linewidth(1.5)

ax_01.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.set_xticks(rho_ticks)
ax_01.set_xticklabels(rho_labels)
#Labels
ax_01.set_xlabel(r'', fontsize=0)
ax_01.set_ylabel(r'Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$', fontsize=my_fontsize-2)
#Limits
ax_01.set_xlim(rho_min, rho_max)
ax_01.set_ylim(rate_min, rate_max)

ax_02 = fig.add_subplot(gs[1,:])
ax_02.patch.set_facecolor('white')
#Plot
for i in range(N_plot):
    for j in np.arange(5):
        if (not(np.all(np.isnan(drift[i,:,j])))):
            #Identify stable zones
            filter_stable = growth_rate[i,:,j] < 0.
            #Plot stable parts
            stable_zones, N_stable = ndi.label(ndi.binary_dilation(filter_stable))
            stable_slices = ndi.find_objects(stable_zones, max_label=N_stable)
            for k in range(N_stable):
                loc = stable_slices[k]
                ax_02.plot(rho[loc], drift[i,:,j][loc], linewidth = my_lw[i], color=cmap_list[i], linestyle=':')
            
            #Plot unstable parts
            unstable_zones, N_unstable = ndi.label(ndi.binary_dilation(np.logical_not(filter_stable)))
            unstable_slices = ndi.find_objects(unstable_zones, max_label=N_unstable)
            for k in range(N_unstable):
                loc = unstable_slices[k]
                ax_02.plot(rho[loc], drift[i,:,j][loc], linewidth = my_lw[i], color=cmap_list[i], linestyle='-')

#Axes
for axis in ['top','bottom','left','right']:
	ax_02.spines[axis].set_linewidth(1.5)

ax_02.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize+2)
ax_02.set_xticks(rho_ticks)
ax_02.set_xticklabels(rho_labels)
ax_02.set_yticks(alpha_ticks)
ax_02.set_yticklabels(alpha_labels)
#Labels
ax_02.set_xlabel(r'Mode mixity ratio $K_\mathrm{III}^0/K_\mathrm{I}^0$', fontsize=my_fontsize)
ax_02.set_ylabel(r'Drift angle $\alpha$', fontsize=my_fontsize)
#Limits
ax_02.set_xlim(rho_min, rho_max)
ax_02.set_ylim(drift_min, drift_max)
#Colorbar
ax_cbar = fig.add_subplot(gs[2,:])
cbar = fig.colorbar(cm.ScalarMappable(norm=cmap_norm, cmap=cmap_plot), cax=ax_cbar, orientation='horizontal', format=cmap_fmt, ticks=cmap_ticks)
cbar.set_label(r'Size ratio $\epsilon_{xx} \cdot \ell_\mathrm{pert}/\mathcal{L}_{xx}$', fontsize=my_fontsize)
cbar.ax.tick_params(labelsize=my_labelsize, width=1.5) 
cbar.outline.set_linewidth(1.5)
cbar.set_alpha(1)
cbar.draw_all()
##Fix position
#Size coefficients
c_y = 0.1
#Interlines
L_x = 0.175
l_x = 0.05
l_y = 0.07
L_y = 0.09
#Graphic sizes
H_x = (0.98-L_x-l_x)
H_y = (0.97-2*L_y-2.25*l_y)/(2+c_y)
C_x = H_x
C_y = c_y*H_y
#Set position
ax_01.set_position([0.01+1*L_x, 0.01+1*H_y+1*C_y+2*L_y+2.25*l_y, H_x, H_y])
ax_02.set_position([0.01+1*L_x, 0.01+0*H_y+1*C_y+2*L_y+1.25*l_y, H_x, H_y])
ax_cbar.set_position([0.01+1*L_x, 0.01+0*H_y+0*C_y+1*L_y+0*l_y, C_x, C_y])
##Save
fig.savefig('figures/EJM_TXX_InstabilityModes.pdf', format='pdf')
plt.close()

#Data for Zenodo
rho_grid, L_pert_over_L_xx_grid = np.meshgrid(rho, L_pert_over_L_xx)
export_header = r"Poisson's ratio $\nu$; Size ratio $\epsilon_{xx} \cdot \ell_\mathrm{pert}/\mathcal{L}_{xx}$; Mode mixity ratio $K_\mathrm{III}^0/K_\mathrm{I}^0$; Sol. n°1 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°1 Drift angle $\alpha$; Sol. n°2 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°2 Drift angle $\alpha$; Sol. n°3 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°3 Drift angle $\alpha$; Sol. n°4 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°4 Drift angle $\alpha$; Sol. n°5 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°5 Drift angle $\alpha$"
export_table = np.column_stack([np.full(rho_grid.shape, nu).flatten(), L_pert_over_L_xx_grid.flatten(), rho_grid.flatten(), growth_rate[:,:,0].flatten(), drift[:,:,0].flatten(), growth_rate[:,:,1].flatten(), drift[:,:,1].flatten(), growth_rate[:,:,2].flatten(), drift[:,:,2].flatten(), growth_rate[:,:,3].flatten(), drift[:,:,3].flatten(), growth_rate[:,:,4].flatten(), drift[:,:,4].flatten()])
np.savetxt("data/data_Figure_3.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')

########################################################################
#Critical mode mixity ratio for several Poisson's ratio
print("I.b T_xx - Stability analysis for several Poisson's ratios")

##Trial grid
#Poisson ratio
nu_trial = np.linspace(0.1, 0.5, num=9, endpoint=True)
#Mode mixity
rho_trial = np.linspace(0., 2., num=101, endpoint=True)
#T-stress
L_pert_over_L_xx_trial = np.linspace(-1., 2., num=101, endpoint=True)
q_xx_trial = np.sign(L_pert_over_L_xx_trial)*np.sqrt(np.abs(L_pert_over_L_xx_trial)/2./np.pi)

#Find xi on the trial grid
Re_xi_trial = np.full((nu_trial.size, q_xx_trial.size, rho_trial.size, 5), np.nan)
Im_xi_trial = np.full((nu_trial.size, q_xx_trial.size, rho_trial.size, 5), np.nan)
for i in range(nu_trial.size):
    for j in range(q_xx_trial.size):
        print("Pre-computation on a grid: {:2d}%".format(int(100.*(i*(q_xx_trial.size)+j+1)/(nu_trial.size*q_xx_trial.size))), end='\r', flush=True)
        for k in range(rho_trial.size):
            #Find xi
            xi = find_xi(rho_trial[k], 1, q_xx_trial[j], 0., 0., nu_trial[i], False)
            Re_xi_trial[i,j,k,:xi.size] =  np.copy(xi.real)
            Im_xi_trial[i,j,k,:xi.size] =  np.copy(xi.imag)

#Find rho_cr on the trial grid
rho_cr_trial = np.zeros((nu_trial.size,q_xx_trial.size))
for i in range(nu_trial.size):
    for j in range(q_xx_trial.size):
        #Filter full NaN columns
        not_nan = np.logical_not(np.all(np.isnan(Re_xi_trial[i,j]), axis=1))
        max_Re = np.full(rho_trial.size, np.nan)
        if (np.any(not_nan)):
            max_Re[not_nan] = np.nanmax(Re_xi_trial[i,j][not_nan], axis=1)
        
        #Check stability
        is_unstable = max_Re >= 0.
        if np.any(is_unstable):
            k_max = np.argmax(is_unstable)
            if (k_max==0):
                #The system is instable for all 
                rho_cr_trial[i,j] = 0.
            else:
                #Limits
                rho_inf = rho_trial[k_max-1]
                rho_sup = rho_trial[k_max]
                Re_xi_inf = max_Re[k_max-1]
                Re_xi_sup = max_Re[k_max]
                #First guess from linear interpolation
                rho_guess = rho_inf + (0.-Re_xi_inf)/(Re_xi_sup-Re_xi_inf)*(rho_sup-rho_inf)
                #Function
                g_rate = lambda rho : max_Re_xi(rho, 1, q_xx_trial[j], 0., 0., nu_trial[i])
                #Find rho_cr so for which one wavelength is unstable
                sol_rho = optim.root(g_rate, rho_guess, tol=1e-5, options=[('maxfev', 1000)])
                if (sol_rho.success):
                    rho_cr_trial[i,j] = sol_rho.x[0]
                else:
                    print("Be careful nu={:.2f}/r={:.3f}. The root finding algorithm did not converge - residual={:.2e}".format(nu_trial[i], L_pert_over_L_xx_trial[j], sol_rho.fun))
                    print("Please increase the number of function evaluations.")
                    rho_cr_trial[i,j] = np.nan
        else:
            rho_cr_trial[i,j] = np.nan

print("\n----- Completed -----")
#Interpolation of rho_cr on the trial grid
guess_rho_cr = interpolate.interp2d(L_pert_over_L_xx_trial, nu_trial, rho_cr_trial, kind='linear')

##Denser grids
#Poisson ratio
nu = np.linspace(nu_trial.min(), nu_trial.max(), num=41, endpoint=True)
#T-stress
L_pert_over_L_xx = np.linspace(L_pert_over_L_xx_trial.min(), L_pert_over_L_xx_trial.max(), num=1001, endpoint=True)
q_xx = np.sign(L_pert_over_L_xx)*np.sqrt(np.abs(L_pert_over_L_xx)/2./np.pi)
#Critical rho
rho_cr = np.zeros((nu.size,q_xx.size))
for i in range(nu.size):
    print("Computation: {:2d}%".format(int(100.*(i*(q_xx.size)+j+1)/(nu.size*q_xx.size))), end='\r', flush=True)
    for j in range(q_xx.size):
        #Function
        g_rate = lambda rho : max_Re_xi(rho, 1, q_xx[j], 0., 0., nu[i])
        #Test if there is an instability for rho = 0
        if (g_rate(0.)>=0.):
            rho_cr[i,j]=0.
        elif (np.isnan(guess_rho_cr(L_pert_over_L_xx[j], nu[i]))):
            rho_cr[i,j] = np.nan
        else :
            #Find rho_cr so for which the wavelength is unstable
            sol_rho = optim.root(g_rate, guess_rho_cr(L_pert_over_L_xx[j], nu[i]), tol=1e-5, options=[('maxfev', 1000)])
            if (sol_rho.success):
                rho_cr[i,j] = sol_rho.x[0]
            else:
                print("Be careful nu={:.2f}/r={:.3f}. The root finding algorithm did not converge - residual={:.2e}".format(nu[i], L_pert_over_L_xx[j], sol_rho.fun))
                print("Please increase the number of function evaluations.")
                rho_cr[i,j] = np.nan

#Critical q-ratio
q_cr_xx = np.zeros(nu.size)
for i in range(nu.size):
    g_rate = lambda q : max_Re_xi(0., 1, q, 0., 0., nu[i])
    q_guess = q_xx[np.argmax(rho_cr[i]==0.)]
    sol_l = optim.root(g_rate, q_guess, tol=1e-5, options=[('maxfev', 1000)])
    if (sol_l.success):
        q_cr_xx[i] = sol_l.x[0]
    else:
        print("Be careful nu={:.2f}. The root finding algorithm did not converge - residual={:.2e}".format(nu[i], sol_l.fun))
        print("Please increase the number of function evaluations.")
        q_cr_xx[i] = np.nan

#Critical wave-length ratio
L_cr_over_L_xx = 2.*np.pi*q_cr_xx**2
print("\n----- Completed -----")

#Indices to plot
nu_plot = np.array([0.1, 0.2, 0.3, 0.4, 0.5])
indices_plot = [np.argmin(np.abs(nu-x)) for x in nu_plot]
N_plot = nu_plot.size
#Plot parameters
my_fontsize = 18
my_labelsize = 14
my_lw = 1.5
#Axis parameters
L_min, L_max = L_pert_over_L_xx.min(), L_pert_over_L_xx.max()
rho_min, rho_max = -0.05, 1.2
rho_ticks = np.arange(0., rho.max()+0.01, step=0.2)
rho_labels = [r'${0:g}$'.format(rho) for rho in rho_ticks]
L_cr_min, L_cr_max = 0.5, 2.
L_cr_ticks = np.arange(L_cr_min, L_cr_max+0.01, step=0.5)
L_cr_labels = [r'${0:g}$'.format(L_cr) for L_cr in L_cr_ticks]
nu_min, nu_max = 0.05, 0.55
nu_ticks = np.arange(0.1, 0.5+0.01, step=0.1)
nu_labels = [r'${0:g}$'.format(nu) for nu in nu_ticks]
#Create custom colormap for nu
cmap_list = [cm_blue, cm_greenblue, cm_black, cm_lightbrown, cm_brown]
#Colormap
cmap_plot = colors.LinearSegmentedColormap.from_list('Discrete cmap', cmap_list, nu_plot.size)
#Labels
cmap_labels = [r'${:.1f}$'.format(x) for x in nu_plot]
N_labels = len(cmap_labels)
#Norm
norm_bins = np.arange(nu_plot.size)+0.5
norm_bins = np.insert(norm_bins, 0, np.min(norm_bins) - 1.0)
norm_diff = norm_bins[1:] - norm_bins[:-1]
cmap_ticks = norm_bins[:-1] + norm_diff / 2
cmap_norm = colors.BoundaryNorm(norm_bins, N_labels, clip=True)
cmap_fmt = ticker.FuncFormatter(lambda x, pos: cmap_labels[cmap_norm(x)])

#Figure
fig = plt.figure(figsize=(4,6))
gs = GridSpec(3, 1, figure=fig)

ax_01 = fig.add_subplot(gs[1,:])
ax_01.patch.set_facecolor('white')
#Plot
rho_plot = LineCollection([np.column_stack([L_pert_over_L_xx, rho_cr[i]]) for i in indices_plot], cmap=cmap_plot, linewidths=my_lw, linestyles='-')
rho_plot.set_array(np.arange(N_plot))
ax_01.add_collection(rho_plot)
#Axes
for axis in ['top','bottom','left','right']:
	ax_01.spines[axis].set_linewidth(1.5)

ax_01.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.set_yticks(rho_ticks)
ax_01.set_yticklabels(rho_labels)
#Labels
ax_01.set_xlabel(r'Size ratio $\epsilon_{xx} \cdot \ell_\mathrm{pert}/\mathcal{L}_{xx}$', fontsize=my_fontsize)
ax_01.set_ylabel(r'Critical mode mixity'+'\n'+r'ratio $\left[K_\mathrm{III}^0/K_\mathrm{I}^0\right]_\mathrm{cr}$', fontsize=my_fontsize)
#Limits
ax_01.set_xlim(L_min, L_max)
ax_01.set_ylim(rho_min, rho_max)
#Colorbar
ax_cbar = fig.add_subplot(gs[0,:])
cbar = fig.colorbar(cm.ScalarMappable(norm=cmap_norm, cmap=cmap_plot), cax=ax_cbar, orientation='horizontal', format=cmap_fmt, ticks=cmap_ticks)
cbar.ax.set_title(r"Poisson's ratio $\nu$", fontsize=my_fontsize)
cbar.ax.tick_params(labelsize=my_labelsize, width=1.5) 
cbar.outline.set_linewidth(1.5)
cbar.set_alpha(1)
cbar.draw_all()

ax_02 = fig.add_subplot(gs[2,:])
ax_02.patch.set_facecolor('white')
#Plot
ax_02.plot(nu, L_cr_over_L_xx, color='black', linewidth=2.)
#Axes
for axis in ['top','bottom','left','right']:
	ax_02.spines[axis].set_linewidth(1.5)

ax_02.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.set_xticks(nu_ticks)
ax_02.set_xticklabels(nu_labels)
ax_02.set_yticks(L_cr_ticks)
ax_02.set_yticklabels(L_cr_labels)
#Labels
ax_02.set_xlabel(r"Poisson's ratio $\nu$", fontsize=my_fontsize)
ax_02.set_ylabel(r'Critical size ratio'+'\n'+r'$\left[\ell_\mathrm{pert}/\mathcal{L}_{xx}\right]_\mathrm{cr}$', fontsize=my_fontsize)
#Limits
ax_02.set_xlim(nu_min, nu_max)
ax_02.set_xlim(nu_min, nu_max)

##Fix position
#Size coefficients
c_y = 0.1
n_y = 0.7
#Interlines
L_x = 0.25
l_x = 0.05
l_y = 0.03
L_y = 0.06
#Graphic sizes
H_x = (0.98-L_x-l_x)
H_y = (0.98-3*L_y-5*l_y)/(1+n_y+c_y)
C_x = H_x
C_y = c_y*H_y
N_x = H_x
N_y = n_y*H_y
#Set position
ax_cbar.set_position([0.01+1*L_x, 0.01+1*H_y+1*N_y+2*L_y+5*l_y, C_x, C_y])
ax_01.set_position([0.01+1*L_x, 0.01+0*H_y+1*N_y+2*L_y+3*l_y, H_x, H_y])
ax_02.set_position([0.01+1*L_x, 0.01+0*H_y+0*N_y+1*L_y+1*l_y, N_x, N_y])
##Save
fig.savefig('figures/EJM_TXX_CriticalModeMixity.pdf', format='pdf')
plt.close()

#Data for Zenodo
L_pert_over_L_xx_grid, nu_grid = np.meshgrid(L_pert_over_L_xx, nu[indices_plot])
export_header = r"Poisson's ratio $\nu$; Size ratio $\epsilon_{xx} \cdot \ell_\mathrm{pert}/\mathcal{L}_{xx}$; Critical mode mixity ratio $\left[K_\mathrm{III}^0/K_\mathrm{I}^0\right]_\mathrm{cr}$"
export_table = np.column_stack([nu_grid.flatten(), L_pert_over_L_xx_grid.flatten(), rho_cr[indices_plot].flatten()])
np.savetxt("data/data_Figure_4a.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')
export_header = r"Poisson's ratio $\nu$; Critical size ratio $\left[\ell_\mathrm{pert}/\mathcal{L}_{xx}\right]_\mathrm{cr}$"
export_table = np.column_stack([nu, L_cr_over_L_xx])
np.savetxt("data/data_Figure_4b.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')

########################################################################
##Influence of T_{zz}
########################################################################
print("--- Part I: Influence of T_zz ---")

########################################################################
#Configuration stability for a fixed Poisson's ratio

#Poisson's ratio
nu = 0.3
print("I.a T_zz - Stability analysis for a Poisson's ratio nu={:.1f}".format(nu))

#Mode mixity
rho = np.linspace(0., 2., num=2001, endpoint=True)
#T-stress
L_pert_over_L_zz = np.array([-1., -0.5, 0., 0.5, 1., 1.5])
q_zz = np.sign(L_pert_over_L_zz)*np.sqrt(np.abs(L_pert_over_L_zz)/2./np.pi)
#Find xi
Re_xi = np.full((q_zz.size, rho.size, 5), np.nan)
Im_xi = np.full((q_zz.size, rho.size, 5), np.nan)
for i in range(q_zz.size):
    print("Computation: {:2d}%".format(int(100.*(i+1)/q_zz.size)), end='\r', flush=True)
    for j in range(rho.size):
        #Find xi
        xi = find_xi(rho[j], 1, 0., 0., q_zz[i], nu, True)
        if (xi.size==2):
            Re_xi[i,j,1:xi.size+1] =  np.copy(xi.real)
            Im_xi[i,j,1:xi.size+1] =  np.copy(xi.imag)
        elif ((xi.size==1) | (xi.size==3)):
            Re_xi[i,j,:xi.size] =  np.copy(xi.real)
            Im_xi[i,j,:xi.size] =  np.copy(xi.imag)
        else :
            print('Be careful with the roots sorting. N_roots={:d}>3'.format(xi.size))
            break

print("\n----- Completed -----")
#Growth rate
growth_rate = Re_xi
#Drift
drift = np.arctan(-Im_xi)
drift[np.isclose(drift, 0.)] = 0.

#Indices to plot
N_plot = L_pert_over_L_zz.size
indices_plot = np.arange(N_plot)
#Plot parameters
my_fontsize = 18
my_labelsize = 14
my_ms = 1
my_lw = [1.,1.,1.5,1.,1.,1.]
#Axis parameters
rho_min, rho_max = 0., rho.max()
rho_ticks = np.arange(0., rho.max()+0.1, step=0.5)
rho_labels = [r'${0:g}$'.format(rho) for rho in rho_ticks]
rate_min, rate_max = -1., 4.
drift_min, drift_max = -np.pi/2., np.pi/2.
alpha_ticks = [-np.pi/2.,-np.pi/4.,0.,np.pi/4.,np.pi/2.]
alpha_labels = [r'-$\frac{\pi}{2}$',r'-$\frac{\pi}{4}$', r'$0$', r'$\frac{\pi}{4}$', r'$\frac{\pi}{2}$']
#Create custom colormap for \ell_\mathrm{pert}/L_zz
cmap_list = [cm_blue, cm_greenblue, cm_black, cm_lightbrown, cm_orange, cm_brown]
#Colormap
cmap_plot = colors.LinearSegmentedColormap.from_list('Discrete cmap', cmap_list, L_pert_over_L_zz.size)
#Labels
cmap_labels = [r'${0:g}$'.format(L) for L in L_pert_over_L_zz]
N_labels = len(cmap_labels)
#Norm
norm_bins = np.arange(L_pert_over_L_zz.size)+0.5
norm_bins = np.insert(norm_bins, 0, np.min(norm_bins) - 1.0)
norm_diff = norm_bins[1:] - norm_bins[:-1]
cmap_ticks = norm_bins[:-1] + norm_diff / 2
cmap_norm = colors.BoundaryNorm(norm_bins, N_labels, clip=True)
cmap_fmt = ticker.FuncFormatter(lambda x, pos: cmap_labels[cmap_norm(x)])

#Figure
fig = plt.figure(figsize=(4,6))
gs = GridSpec(3, 1, figure=fig)

ax_01 = fig.add_subplot(gs[0,:])
ax_01.patch.set_facecolor('white')
#Plot
ax_01.axhspan(rate_min, 0., color='grey', linewidth=0., alpha=0.5, zorder=-1)
ax_01.axhline(0., color='grey', linewidth=0.5, linestyle=':', zorder=-1)
for i in range(N_plot):
    for j in np.arange(5):
        filter_straight = np.isclose(drift[i,:,j],0.)
        filter_drift = np.logical_not(filter_straight)
        ax_01.plot(rho[filter_straight], growth_rate[i,:,j][filter_straight], linewidth = my_lw[i], color=cmap_list[i], linestyle='-')
        ax_01.plot(rho[filter_drift], growth_rate[i,:,j][filter_drift], linewidth = my_lw[i], color=cmap_list[i], linestyle='--')

#Axes
for axis in ['top','bottom','left','right']:
	ax_01.spines[axis].set_linewidth(1.5)

ax_01.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.set_xticks(rho_ticks)
ax_01.set_xticklabels(rho_labels)
#Labels
ax_01.set_xlabel(r'', fontsize=0)
ax_01.set_ylabel(r'Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$', fontsize=my_fontsize-2)
#Limits
ax_01.set_xlim(rho_min, rho_max)
ax_01.set_ylim(rate_min, rate_max)

ax_02 = fig.add_subplot(gs[1,:])
ax_02.patch.set_facecolor('white')
#Plot
for i in range(N_plot):
    for j in np.arange(5):
        if (not(np.all(np.isnan(drift[i,:,j])))):
            #Identify stable zones
            filter_stable = growth_rate[i,:,j] < 0.
            #Plot stable parts
            stable_zones, N_stable = ndi.label(ndi.binary_dilation(filter_stable))
            stable_slices = ndi.find_objects(stable_zones, max_label=N_stable)
            for k in range(N_stable):
                loc = stable_slices[k]
                ax_02.plot(rho[loc], drift[i,:,j][loc], linewidth = my_lw[i], color=cmap_list[i], linestyle=':')
            
            #Plot unstable parts
            unstable_zones, N_unstable = ndi.label(ndi.binary_dilation(np.logical_not(filter_stable)))
            unstable_slices = ndi.find_objects(unstable_zones, max_label=N_unstable)
            for k in range(N_unstable):
                loc = unstable_slices[k]
                ax_02.plot(rho[loc], drift[i,:,j][loc], linewidth = my_lw[i], color=cmap_list[i], linestyle='-')

#Axes
for axis in ['top','bottom','left','right']:
	ax_02.spines[axis].set_linewidth(1.5)

ax_02.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize+2)
ax_02.set_xticks(rho_ticks)
ax_02.set_xticklabels(rho_labels)
ax_02.set_yticks(alpha_ticks)
ax_02.set_yticklabels(alpha_labels)
#Labels
ax_02.set_xlabel(r'Mode mixity ratio $K_\mathrm{III}^0/K_\mathrm{I}^0$', fontsize=my_fontsize)
ax_02.set_ylabel(r'Drift angle $\alpha$', fontsize=my_fontsize)
#Limits
ax_02.set_xlim(rho_min, rho_max)
ax_02.set_ylim(drift_min, drift_max)
#Colorbar
ax_cbar = fig.add_subplot(gs[2,:])
cbar = fig.colorbar(cm.ScalarMappable(norm=cmap_norm, cmap=cmap_plot), cax=ax_cbar, orientation='horizontal', format=cmap_fmt, ticks=cmap_ticks)
cbar.set_label(r'Size ratio $\epsilon_{zz} \cdot \ell_\mathrm{pert}/\mathcal{L}_{zz}$', fontsize=my_fontsize)
cbar.ax.tick_params(labelsize=my_labelsize, width=1.5) 
cbar.outline.set_linewidth(1.5)
cbar.set_alpha(1)
cbar.draw_all()
##Fix position
#Size coefficients
c_y = 0.1
#Interlines
L_x = 0.175
l_x = 0.05
l_y = 0.07
L_y = 0.09
#Graphic sizes
H_x = (0.98-L_x-l_x)
H_y = (0.97-2*L_y-2.25*l_y)/(2+c_y)
C_x = H_x
C_y = c_y*H_y
#Set position
ax_01.set_position([0.01+1*L_x, 0.01+1*H_y+1*C_y+2*L_y+2.25*l_y, H_x, H_y])
ax_02.set_position([0.01+1*L_x, 0.01+0*H_y+1*C_y+2*L_y+1.25*l_y, H_x, H_y])
ax_cbar.set_position([0.01+1*L_x, 0.01+0*H_y+0*C_y+1*L_y+0*l_y, C_x, C_y])
##Save
fig.savefig('figures/EJM_TZZ_InstabilityModes.pdf', format='pdf')
plt.close()

#Data for Zenodo
rho_grid, L_pert_over_L_zz_grid = np.meshgrid(rho, L_pert_over_L_zz)
export_header = r"Poisson's ratio $\nu$; Size ratio $\epsilon_{zz} \cdot \ell_\mathrm{pert}/\mathcal{L}_{zz}$; Mode mixity ratio $K_\mathrm{III}^0/K_\mathrm{I}^0$; Sol. n°1 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°1 Drift angle $\alpha$; Sol. n°2 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°2 Drift angle $\alpha$; Sol. n°3 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°3 Drift angle $\alpha$; Sol. n°4 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°4 Drift angle $\alpha$; Sol. n°5 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°5 Drift angle $\alpha$"
export_table = np.column_stack([np.full(rho_grid.shape, nu).flatten(), L_pert_over_L_zz_grid.flatten(), rho_grid.flatten(), growth_rate[:,:,0].flatten(), drift[:,:,0].flatten(), growth_rate[:,:,1].flatten(), drift[:,:,1].flatten(), growth_rate[:,:,2].flatten(), drift[:,:,2].flatten(), growth_rate[:,:,3].flatten(), drift[:,:,3].flatten(), growth_rate[:,:,4].flatten(), drift[:,:,4].flatten()])
np.savetxt("data/data_Figure_5.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')

########################################################################
#Critical mode mixity ratio for several Poisson's ratio
print("I.b T_zz - Stability analysis for several Poisson's ratios")

##Trial grid
#Poisson ratio
nu_trial = np.linspace(0.1, 0.5, num=9, endpoint=True)
#Mode mixity
rho_trial = np.concatenate((np.linspace(0., 2., num=101, endpoint=True), np.arange(2., 10.01, step=0.25)[1:]))
#T-stress
L_pert_over_L_zz_trial = np.concatenate((np.linspace(-1., 2., num=101, endpoint=True), np.logspace(np.log2(2.), np.log2(512.), num=41, endpoint=True, base=2)[1:]))
q_zz_trial = np.sign(L_pert_over_L_zz_trial)*np.sqrt(np.abs(L_pert_over_L_zz_trial)/2./np.pi)

#Find xi on the trial grid
Re_xi_trial = np.full((nu_trial.size, q_zz_trial.size, rho_trial.size, 5), np.nan)
Im_xi_trial = np.full((nu_trial.size, q_zz_trial.size, rho_trial.size, 5), np.nan)
for i in range(nu_trial.size):
    for j in range(q_zz_trial.size):
        print("Pre-computation on a grid: {:2d}%".format(int(100.*(i*(q_zz_trial.size)+j+1)/(nu_trial.size*q_zz_trial.size))), end='\r', flush=True)
        for k in range(rho_trial.size):
            #Find xi
            xi = find_xi(rho_trial[k], 1, 0., 0., q_zz_trial[j], nu_trial[i], False)
            Re_xi_trial[i,j,k,:xi.size] =  np.copy(xi.real)
            Im_xi_trial[i,j,k,:xi.size] =  np.copy(xi.imag)

#Find rho_cr on the trial grid
rho_cr_trial = np.zeros((nu_trial.size,q_zz_trial.size))
for i in range(nu_trial.size):
    for j in range(q_zz_trial.size):
        #Filter full NaN columns
        not_nan = np.logical_not(np.all(np.isnan(Re_xi_trial[i,j]), axis=1))
        max_Re = np.full(rho_trial.size, np.nan)
        if (np.any(not_nan)):
            max_Re[not_nan] = np.nanmax(Re_xi_trial[i,j][not_nan], axis=1)
        
        #Check stability
        is_unstable = max_Re >= 0.
        if np.any(is_unstable):
            k_max = np.argmax(is_unstable)
            if (k_max==0):
                #The system is instable for all 
                rho_cr_trial[i,j] = 0.
            else:
                #Limits
                rho_inf = rho_trial[k_max-1]
                rho_sup = rho_trial[k_max]
                Re_xi_inf = max_Re[k_max-1]
                Re_xi_sup = max_Re[k_max]
                #First guess from linear interpolation
                rho_guess = rho_inf + (0.-Re_xi_inf)/(Re_xi_sup-Re_xi_inf)*(rho_sup-rho_inf)
                #Function
                g_rate = lambda rho : max_Re_xi(rho, 1, 0., 0., q_zz_trial[j], nu_trial[i])
                #Find rho_cr so for which one wavelength is unstable
                sol_rho = optim.root(g_rate, rho_guess, tol=1e-5, options=[('maxfev', 1000)])
                if (sol_rho.success):
                    rho_cr_trial[i,j] = sol_rho.x[0]
                else:
                    print("Be careful nu={:.2f}/r={:.3f}. The root finding algorithm did not converge - residual={:.2e}".format(nu_trial[i], L_pert_over_L_zz_trial[j], sol_rho.fun))
                    print("Please increase the number of function evaluations.")
                    rho_cr_trial[i,j] = np.nan
        else:
            rho_cr_trial[i,j] = np.nan

print("\n----- Completed -----")
#Interpolation of rho_cr on the trial grid
guess_rho_cr = interpolate.interp2d(L_pert_over_L_zz_trial, nu_trial, rho_cr_trial, kind='linear')

##Denser grids
#Poisson ratio
nu = np.linspace(nu_trial.min(), nu_trial.max(), num=41, endpoint=True)
#T-stress
L_pert_over_L_zz = np.concatenate((np.linspace(-1., 2., num=1001, endpoint=True), np.logspace(np.log2(2.), np.log2(512.), num=201, endpoint=True, base=2)[1:]))
q_zz = np.sign(L_pert_over_L_zz)*np.sqrt(np.abs(L_pert_over_L_zz)/2./np.pi)
#Critical rho
rho_cr = np.zeros((nu.size,q_zz.size))
for i in range(nu.size):
    print("Computation: {:2d}%".format(int(100.*(i*(q_zz.size)+j+1)/(nu.size*q_zz.size))), end='\r', flush=True)
    for j in range(q_zz.size):
        #Function
        g_rate = lambda rho : max_Re_xi(rho, 1, 0., 0., q_zz[j], nu[i])
        #Test if there is an instability for rho = 0
        if (g_rate(0.)>=0.):
            rho_cr[i,j]=0.
        elif (np.isnan(guess_rho_cr(L_pert_over_L_zz[j], nu[i]))):
            rho_cr[i,j] = np.nan
        else :
            #Find rho_cr so for which the wavelength is unstable
            sol_rho = optim.root(g_rate, guess_rho_cr(L_pert_over_L_zz[j], nu[i]), tol=1e-5, options=[('maxfev', 1000)])
            if (sol_rho.success):
                rho_cr[i,j] = sol_rho.x[0]
            else:
                print("Be careful nu={:.2f}/r={:.3f}. The root finding algorithm did not converge - residual={:.2e}".format(nu[i], L_pert_over_L_zz[j], sol_rho.fun))
                print("Please increase the number of function evaluations.")
                rho_cr[i,j] = np.nan

#Critical q-ratio
q_cr_zz = np.zeros(nu.size)
for i in range(nu.size):
    g_rate = lambda q : max_Re_xi(0., 1, 0., 0., q, nu[i])
    q_guess = q_zz[np.argmax(rho_cr[i]==0.)]
    sol_l = optim.root(g_rate, q_guess, tol=1e-5, options=[('maxfev', 1000)])
    if (sol_l.success):
        q_cr_zz[i] = sol_l.x[0]
    else:
        print("Be careful nu={:.2f}. The root finding algorithm did not converge - residual={:.2e}".format(nu[i], sol_l.fun))
        print("Please increase the number of function evaluations.")
        q_cr_zz[i] = np.nan

#Critical wave-length ratio
L_cr_over_L_zz = 2.*np.pi*q_cr_zz**2
L_cr_pred = np.pi/4.*(2.-3.*nu)**2/nu**2
print("\n----- Completed -----")

#Indices to plot
nu_plot = np.array([0.1, 0.2, 0.3, 0.4, 0.5])
indices_plot = [np.argmin(np.abs(nu-x)) for x in nu_plot]
N_plot = nu_plot.size
#Plot parameters
my_fontsize = 18
my_labelsize = 14
my_lw = 1.5
#Axis parameters
L_min, L_max = L_pert_over_L_zz.min(), 2.
rho_min, rho_max = -0.05, 1.2
rho_ticks = np.arange(0., rho_trial.max()+0.01, step=0.2)
rho_labels = [r'${0:g}$'.format(rho) for rho in rho_ticks]
L_cr_scale = 'log'
L_cr_min, L_cr_max = 5E-1, 3E2
L_cr_ticks = np.logspace(0.,2., num=3, endpoint=True, base=10)
L_cr_labels = [r'$10^{'+str('%.0f' % (np.log10(L_cr)))+'}$' for L_cr in L_cr_ticks]
# L_cr_scale = 'linear'
# L_cr_min, L_cr_max = 0., 5.
# L_cr_ticks = np.arange(L_cr_min, L_cr_max+0.1, step=1.)
# L_cr_labels = [r'${0:g}$'.format(L_cr) for L_cr in L_cr_ticks]
nu_min, nu_max = 0.05, 0.55
nu_ticks = np.arange(0.1, 0.5+0.01, step=0.1)
nu_labels = [r'${0:g}$'.format(nu) for nu in nu_ticks]
#Create custom colormap for nu
cmap_list = [cm_blue, cm_greenblue, cm_black, cm_lightbrown, cm_brown]
#Colormap
cmap_plot = colors.LinearSegmentedColormap.from_list('Discrete cmap', cmap_list, nu_plot.size)
#Labels
cmap_labels = [r'${:.1f}$'.format(x) for x in nu_plot]
N_labels = len(cmap_labels)
#Norm
norm_bins = np.arange(nu_plot.size)+0.5
norm_bins = np.insert(norm_bins, 0, np.min(norm_bins) - 1.0)
norm_diff = norm_bins[1:] - norm_bins[:-1]
cmap_ticks = norm_bins[:-1] + norm_diff / 2
cmap_norm = colors.BoundaryNorm(norm_bins, N_labels, clip=True)
cmap_fmt = ticker.FuncFormatter(lambda x, pos: cmap_labels[cmap_norm(x)])

#Figure
fig = plt.figure(figsize=(4,6))
gs = GridSpec(3, 1, figure=fig)

ax_01 = fig.add_subplot(gs[1,:])
ax_01.patch.set_facecolor('white')
#Plot
rho_plot = LineCollection([np.column_stack([L_pert_over_L_zz, rho_cr[i]]) for i in indices_plot], cmap=cmap_plot, linewidths=my_lw, linestyles='-')
rho_plot.set_array(np.arange(N_plot))
ax_01.add_collection(rho_plot)
#Axes
for axis in ['top','bottom','left','right']:
	ax_01.spines[axis].set_linewidth(1.5)

ax_01.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.set_yticks(rho_ticks)
ax_01.set_yticklabels(rho_labels)
#Labels
ax_01.set_xlabel(r'Size ratio $\epsilon_{zz} \cdot \ell_\mathrm{pert}/\mathcal{L}_{zz}$', fontsize=my_fontsize)
ax_01.set_ylabel(r'Critical mode mixity'+'\n'+r'ratio $\left[K_\mathrm{III}^0/K_\mathrm{I}^0\right]_\mathrm{cr}$', fontsize=my_fontsize)
#Limits
ax_01.set_xlim(L_min, L_max)
ax_01.set_ylim(rho_min, rho_max)
#Colorbar
ax_cbar = fig.add_subplot(gs[0,:])
cbar = fig.colorbar(cm.ScalarMappable(norm=cmap_norm, cmap=cmap_plot), cax=ax_cbar, orientation='horizontal', format=cmap_fmt, ticks=cmap_ticks)
cbar.ax.set_title(r"Poisson's ratio $\nu$", fontsize=my_fontsize)
cbar.ax.tick_params(labelsize=my_labelsize, width=1.5) 
cbar.outline.set_linewidth(1.5)
cbar.set_alpha(1)
cbar.draw_all()

ax_02 = fig.add_subplot(gs[2,:])
ax_02.patch.set_facecolor('white')
#Plot
ax_02.plot(nu, L_cr_over_L_zz, color='black', linewidth=2.)
ax_02.plot(nu, L_cr_pred, color='darkorange', linewidth=2., linestyle=(0, (5, 5)))
#Axes
for axis in ['top','bottom','left','right']:
	ax_02.spines[axis].set_linewidth(1.5)

ax_02.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.tick_params(axis='y', which='minor', direction='out', length=2, width=0.5, labelsize=my_labelsize)
ax_02.set_yscale(L_cr_scale)
ax_02.set_xticks(nu_ticks)
ax_02.set_xticklabels(nu_labels)
ax_02.set_yticks(L_cr_ticks)
ax_02.set_yticklabels(L_cr_labels)
#Labels
ax_02.set_xlabel(r"Poisson's ratio $\nu$", fontsize=my_fontsize)
ax_02.set_ylabel(r'Critical size ratio'+'\n'+r'$\left[\ell_\mathrm{pert}/\mathcal{L}_{zz}\right]_\mathrm{cr}$', fontsize=my_fontsize)
#Limits
ax_02.set_xlim(nu_min, nu_max)
ax_02.set_ylim(L_cr_min, L_cr_max)

##Fix position
#Size coefficients
c_y = 0.1
n_y = 0.7
#Interlines
L_x = 0.25
l_x = 0.05
l_y = 0.03
L_y = 0.06
#Graphic sizes
H_x = (0.98-L_x-l_x)
H_y = (0.98-3*L_y-5*l_y)/(1+n_y+c_y)
C_x = H_x
C_y = c_y*H_y
N_x = H_x
N_y = n_y*H_y
#Set position
ax_cbar.set_position([0.01+1*L_x, 0.01+1*H_y+1*N_y+2*L_y+5*l_y, C_x, C_y])
ax_01.set_position([0.01+1*L_x, 0.01+0*H_y+1*N_y+2*L_y+3*l_y, H_x, H_y])
ax_02.set_position([0.01+1*L_x, 0.01+0*H_y+0*N_y+1*L_y+1*l_y, N_x, N_y])
##Save
fig.savefig('figures/EJM_TZZ_CriticalModeMixity.pdf', format='pdf')
plt.close()

#Data for Zenodo
L_pert_over_L_zz_grid, nu_grid = np.meshgrid(L_pert_over_L_zz, nu[indices_plot])
export_header = r"Poisson's ratio $\nu$; Size ratio $\epsilon_{zz} \cdot \ell_\mathrm{pert}/\mathcal{L}_{zz}$; Critical mode mixity ratio $\left[K_\mathrm{III}^0/K_\mathrm{I}^0\right]_\mathrm{cr}$"
export_table = np.column_stack([nu_grid.flatten(), L_pert_over_L_zz_grid.flatten(), rho_cr[indices_plot].flatten()])
np.savetxt("data/data_Figure_6a.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')
export_header = r"Poisson's ratio $\nu$; Critical size ratio $\left[\ell_\mathrm{pert}/\mathcal{L}_{zz}\right]_\mathrm{cr}$"
export_table = np.column_stack([nu, L_cr_over_L_zz])
np.savetxt("data/data_Figure_6b.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')

########################################################################
##Influence of T_{xz}
########################################################################
print("--- Part I: Influence of T_xz ---")

########################################################################
#Configuration stability for a fixed Poisson's ratio

#Poisson's ratio
nu = 0.3
print("I.a T_xz - Stability analysis for a Poisson's ratio nu={:.1f}".format(nu))

#Mode mixity
rho = np.linspace(0., 2., num=2001, endpoint=True)
#T-stress
L_pert_over_L_xz = np.array([-1., -0.5, -0.2, 0., 0.2, 0.5, 1.])
q_xz = np.sign(L_pert_over_L_xz)*np.sqrt(np.abs(L_pert_over_L_xz)/2./np.pi)
#Find xi
Re_xi = np.full((q_xz.size, rho.size, 5), np.nan)
Im_xi = np.full((q_xz.size, rho.size, 5), np.nan)
for i in range(q_xz.size):
    print("Computation: {:2d}%".format(int(100.*(i+1)/q_xz.size)), end='\r', flush=True)
    for j in range(rho.size):
        #Find xi
        xi = find_xi(rho[j], 1, 0., q_xz[i], 0., nu, True)
        if (xi.size==2):
            Re_xi[i,j,1:xi.size+1] =  np.copy(xi.real)
            Im_xi[i,j,1:xi.size+1] =  np.copy(xi.imag)
        elif ((xi.size==1) | (xi.size==3)):
            Re_xi[i,j,:xi.size] =  np.copy(xi.real)
            Im_xi[i,j,:xi.size] =  np.copy(xi.imag)
        else :
            print('Be careful with the roots sorting. N_roots={:d}>3'.format(xi.size))
            break

print("\n----- Completed -----")
#Growth rate
growth_rate = Re_xi
#Drift
drift = np.arctan(-Im_xi)
drift[np.isclose(drift, 0.)] = 0.

#Indices to plot
N_plot = L_pert_over_L_xz.size
indices_plot = np.arange(N_plot)
#Plot parameters
my_fontsize = 18
my_labelsize = 14
my_ms = 1
my_lw = [1.,1.,1.,1.5,1.,1.,1.]
#Axis parameters
rho_min, rho_max = 0., rho.max()
rho_ticks = np.arange(0., rho.max()+0.1, step=0.5)
rho_labels = [r'${0:g}$'.format(rho) for rho in rho_ticks]
rate_min, rate_max = -1., 4.
drift_min, drift_max = -np.pi/2., np.pi/2.
alpha_ticks = [-np.pi/2.,-np.pi/4.,0.,np.pi/4.,np.pi/2.]
alpha_labels = [r'-$\frac{\pi}{2}$',r'-$\frac{\pi}{4}$', r'$0$', r'$\frac{\pi}{4}$', r'$\frac{\pi}{2}$']
#Create custom colormap for \ell_\mathrm{pert}/L_xz
cmap_list = [cm_darkblue, cm_blue, cm_greenblue, cm_black, cm_lightbrown, cm_orange, cm_brown]
#Colormap
cmap_plot = colors.LinearSegmentedColormap.from_list('Discrete cmap', cmap_list, L_pert_over_L_xz.size)
#Labels
cmap_labels = [r'${0:g}$'.format(L) for L in L_pert_over_L_xz]
N_labels = len(cmap_labels)
#Norm
norm_bins = np.arange(L_pert_over_L_xz.size)+0.5
norm_bins = np.insert(norm_bins, 0, np.min(norm_bins) - 1.0)
norm_diff = norm_bins[1:] - norm_bins[:-1]
cmap_ticks = norm_bins[:-1] + norm_diff / 2
cmap_norm = colors.BoundaryNorm(norm_bins, N_labels, clip=True)
cmap_fmt = ticker.FuncFormatter(lambda x, pos: cmap_labels[cmap_norm(x)])

#Figure
fig = plt.figure(figsize=(4,6))
gs = GridSpec(3, 1, figure=fig)

ax_01 = fig.add_subplot(gs[0,:])
ax_01.patch.set_facecolor('white')
#Plot
ax_01.axhspan(rate_min, 0., color='grey', linewidth=0., alpha=0.5, zorder=-1)
ax_01.axhline(0., color='grey', linewidth=0.5, linestyle=':', zorder=-1)
for i in range(N_plot):
    for j in np.arange(5):
        filter_straight = np.isclose(drift[i,:,j],0.)
        filter_drift = np.logical_not(filter_straight)
        ax_01.plot(rho[filter_straight], growth_rate[i,:,j][filter_straight], linewidth = my_lw[i], color=cmap_list[i], linestyle='-')
        ax_01.plot(rho[filter_drift], growth_rate[i,:,j][filter_drift], linewidth = my_lw[i], color=cmap_list[i], linestyle=(0, (8, 8)) if (L_pert_over_L_xz[i]>0.) else '-')

#Axes
for axis in ['top','bottom','left','right']:
	ax_01.spines[axis].set_linewidth(1.5)

ax_01.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.set_xticks(rho_ticks)
ax_01.set_xticklabels(rho_labels)
#Labels
ax_01.set_xlabel(r'', fontsize=0)
ax_01.set_ylabel(r'Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$', fontsize=my_fontsize-2)
#Limits
ax_01.set_xlim(rho_min, rho_max)
ax_01.set_ylim(rate_min, rate_max)

ax_02 = fig.add_subplot(gs[1,:])
ax_02.patch.set_facecolor('white')
#Plot
for i in range(N_plot):
    for j in np.arange(5):
        if (not(np.all(np.isnan(drift[i,:,j])))):
            #Identify stable zones
            filter_stable = growth_rate[i,:,j] < 0.
            #Plot stable parts
            stable_zones, N_stable = ndi.label(ndi.binary_dilation(filter_stable))
            stable_slices = ndi.find_objects(stable_zones, max_label=N_stable)
            for k in range(N_stable):
                loc = stable_slices[k]
                ax_02.plot(rho[loc], drift[i,:,j][loc], linewidth = my_lw[i], color=cmap_list[i], linestyle=':')
            
            #Plot unstable parts
            unstable_zones, N_unstable = ndi.label(ndi.binary_dilation(np.logical_not(filter_stable)))
            unstable_slices = ndi.find_objects(unstable_zones, max_label=N_unstable)
            for k in range(N_unstable):
                loc = unstable_slices[k]
                ax_02.plot(rho[loc], drift[i,:,j][loc], linewidth = my_lw[i], color=cmap_list[i], linestyle='-')

#Axes
for axis in ['top','bottom','left','right']:
	ax_02.spines[axis].set_linewidth(1.5)

ax_02.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize+2)
ax_02.set_xticks(rho_ticks)
ax_02.set_xticklabels(rho_labels)
ax_02.set_yticks(alpha_ticks)
ax_02.set_yticklabels(alpha_labels)
#Labels
ax_02.set_xlabel(r'Mode mixity ratio $K_\mathrm{III}^0/K_\mathrm{I}^0$', fontsize=my_fontsize)
ax_02.set_ylabel(r'Drift angle $\alpha$', fontsize=my_fontsize)
#Limits
ax_02.set_xlim(rho_min, rho_max)
ax_02.set_ylim(drift_min, drift_max)
#Colorbar
ax_cbar = fig.add_subplot(gs[2,:])
cbar = fig.colorbar(cm.ScalarMappable(norm=cmap_norm, cmap=cmap_plot), cax=ax_cbar, orientation='horizontal', format=cmap_fmt, ticks=cmap_ticks)
cbar.set_label(r'Size ratio $\epsilon_{xz} \cdot \ell_\mathrm{pert}/\mathcal{L}_{xz}$', fontsize=my_fontsize)
cbar.ax.tick_params(labelsize=my_labelsize, width=1.5) 
cbar.outline.set_linewidth(1.5)
cbar.set_alpha(1)
cbar.draw_all()
##Fix position
#Size coefficients
c_y = 0.1
#Interlines
L_x = 0.175
l_x = 0.05
l_y = 0.07
L_y = 0.09
#Graphic sizes
H_x = (0.98-L_x-l_x)
H_y = (0.97-2*L_y-2.25*l_y)/(2+c_y)
C_x = H_x
C_y = c_y*H_y
#Set position
ax_01.set_position([0.01+1*L_x, 0.01+1*H_y+1*C_y+2*L_y+2.25*l_y, H_x, H_y])
ax_02.set_position([0.01+1*L_x, 0.01+0*H_y+1*C_y+2*L_y+1.25*l_y, H_x, H_y])
ax_cbar.set_position([0.01+1*L_x, 0.01+0*H_y+0*C_y+1*L_y+0*l_y, C_x, C_y])
##Save
fig.savefig('figures/EJM_TXZ_InstabilityModes.pdf', format='pdf')
plt.close()

#Data for Zenodo
rho_grid, L_pert_over_L_xz_grid = np.meshgrid(rho, L_pert_over_L_xz)
export_header = r"Poisson's ratio $\nu$; Size ratio $\epsilon_{xz} \cdot \ell_\mathrm{pert}/\mathcal{L}_{xz}$; Mode mixity ratio $K_\mathrm{III}^0/K_\mathrm{I}^0$; Sol. n°1 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°1 Drift angle $\alpha$; Sol. n°2 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°2 Drift angle $\alpha$; Sol. n°3 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°3 Drift angle $\alpha$; Sol. n°4 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°4 Drift angle $\alpha$; Sol. n°5 Growth rate $\mathrm{Re}\left(\lambda\right)/\vert k \vert$; Sol. n°5 Drift angle $\alpha$"
export_table = np.column_stack([np.full(rho_grid.shape, nu).flatten(), L_pert_over_L_xz_grid.flatten(), rho_grid.flatten(), growth_rate[:,:,0].flatten(), drift[:,:,0].flatten(), growth_rate[:,:,1].flatten(), drift[:,:,1].flatten(), growth_rate[:,:,2].flatten(), drift[:,:,2].flatten(), growth_rate[:,:,3].flatten(), drift[:,:,3].flatten(), growth_rate[:,:,4].flatten(), drift[:,:,4].flatten()])
np.savetxt("data/data_Figure_7.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')

########################################################################
#Critical mode mixity ratio for several Poisson's ratio
print("I.b T_xz - Stability analysis for several Poisson's ratios")

##Trial grid
#Poisson ratio
nu_trial = np.linspace(0.1, 0.5, num=9, endpoint=True)
#Mode mixity
rho_trial = np.linspace(0., 2., num=101, endpoint=True)
#T-stress
L_pert_over_L_xz_trial = np.concatenate((np.linspace(-5., -1., num=9, endpoint=True)[:-1], np.linspace(-1., 1., num=101, endpoint=True), np.linspace(1., 5., num=9, endpoint=True)[1:]))
q_xz_trial = np.sign(L_pert_over_L_xz_trial)*np.sqrt(np.abs(L_pert_over_L_xz_trial)/2./np.pi)

#Find xi on the trial grid
Re_xi_trial = np.full((nu_trial.size, q_xz_trial.size, rho_trial.size, 5), np.nan)
Im_xi_trial = np.full((nu_trial.size, q_xz_trial.size, rho_trial.size, 5), np.nan)
for i in range(nu_trial.size):
    for j in range(q_xz_trial.size):
        print("Pre-computation on a grid: {:2d}%".format(int(100.*(i*(q_xz_trial.size)+j+1)/(nu_trial.size*q_xz_trial.size))), end='\r', flush=True)
        for k in range(rho_trial.size):
            #Find xi
            xi = find_xi(rho_trial[k], 1, 0., q_xz_trial[j], 0., nu_trial[i], False)
            Re_xi_trial[i,j,k,:xi.size] =  np.copy(xi.real)
            Im_xi_trial[i,j,k,:xi.size] =  np.copy(xi.imag)

#Find rho_cr on the trial grid
rho_cr_trial = np.zeros((nu_trial.size,q_xz_trial.size))
for i in range(nu_trial.size):
    for j in range(q_xz_trial.size):
        #Filter full NaN columns
        not_nan = np.logical_not(np.all(np.isnan(Re_xi_trial[i,j]), axis=1))
        max_Re = np.full(rho_trial.size, np.nan)
        if (np.any(not_nan)):
            max_Re[not_nan] = np.nanmax(Re_xi_trial[i,j][not_nan], axis=1)
        
        #Check stability
        is_unstable = max_Re >= 0.
        if np.any(is_unstable):
            k_max = np.argmax(is_unstable)
            if (k_max==0):
                #The system is instable for all 
                rho_cr_trial[i,j] = 0.
            else:
                #Limits
                rho_inf = rho_trial[k_max-1]
                rho_sup = rho_trial[k_max]
                Re_xi_inf = max_Re[k_max-1]
                Re_xi_sup = max_Re[k_max]
                #First guess from linear interpolation
                rho_guess = rho_inf + (0.-Re_xi_inf)/(Re_xi_sup-Re_xi_inf)*(rho_sup-rho_inf)
                #Function
                g_rate = lambda rho : max_Re_xi(rho, 1, 0., q_xz_trial[j], 0., nu_trial[i])
                #Find rho_cr so for which one wavelength is unstable
                sol_rho = optim.root(g_rate, rho_guess, tol=1e-5, options=[('maxfev', 1000)])
                if (sol_rho.success):
                    rho_cr_trial[i,j] = sol_rho.x[0]
                else:
                    print("Be careful nu={:.2f}/r={:.3f}. The root finding algorithm did not converge - residual={:.2e}".format(nu_trial[i], L_pert_over_L_xz_trial[j], sol_rho.fun))
                    print("Please increase the number of function evaluations.")
                    rho_cr_trial[i,j] = np.nan
        else:
            rho_cr_trial[i,j] = np.nan

print("\n----- Completed -----")
#Interpolation of rho_cr on the trial grid
guess_rho_cr = interpolate.interp2d(L_pert_over_L_xz_trial, nu_trial, rho_cr_trial, kind='linear')

##Denser grids
#Poisson ratio
nu = np.linspace(nu_trial.min(), nu_trial.max(), num=41, endpoint=True)
#T-stress
L_pert_over_L_xz = np.concatenate((np.linspace(-5., -1., num=81, endpoint=True)[:-1], np.linspace(-1., 1., num=1001, endpoint=True), np.linspace(1., 5., num=81, endpoint=True)[1:]))
q_xz = np.sign(L_pert_over_L_xz)*np.sqrt(np.abs(L_pert_over_L_xz)/2./np.pi)
#Critical rho
rho_cr = np.zeros((nu.size,q_xz.size))
for i in range(nu.size):
    print("Computation: {:2d}%".format(int(100.*(i*(q_xz.size)+j+1)/(nu.size*q_xz.size))), end='\r', flush=True)
    for j in range(q_xz.size):
        #Function
        g_rate = lambda rho : max_Re_xi(rho, 1, 0., q_xz[j], 0., nu[i])
        #Test if there is an instability for rho = 0
        if (g_rate(0.)>=0.):
            rho_cr[i,j]=0.
        elif (np.isnan(guess_rho_cr(L_pert_over_L_xz[j], nu[i]))):
            rho_cr[i,j] = np.nan
        else :
            #Find rho_cr so for which the wavelength is unstable
            sol_rho = optim.root(g_rate, guess_rho_cr(L_pert_over_L_xz[j], nu[i]), tol=1e-5, options=[('maxfev', 1000)])
            if (sol_rho.success):
                rho_cr[i,j] = sol_rho.x[0]
            else:
                print("Be careful nu={:.2f}/r={:.3f}. The root finding algorithm did not converge - residual={:.2e}".format(nu[i], L_pert_over_L_xz[j], sol_rho.fun))
                print("Please increase the number of function evaluations.")
                rho_cr[i,j] = np.nan

#Critical q-ratio
q_cr_xz = np.zeros(nu.size)
for i in range(nu.size):
    g_rate = lambda q : max_Re_xi(0., 1, 0., q, 0., nu[i])
    q_guess = q_xz[np.argmax(rho_cr[i]==0.)]
    sol_l = optim.root(g_rate, q_guess, tol=1e-5, options=[('maxfev', 1000)])
    if (sol_l.success):
        q_cr_xz[i] = sol_l.x[0]
    else:
        print("Be careful nu={:.2f}. The root finding algorithm did not converge - residual={:.2e}".format(nu[i], sol_l.fun))
        print("Please increase the number of function evaluations.")
        q_cr_xz[i] = np.nan

#Critical wave-length ratio
L_cr_over_L_xz = 2.*np.pi*q_cr_xz**2
print("\n----- Completed -----")

#Indices to plot
nu_plot = np.array([0.1, 0.2, 0.3, 0.4, 0.5])
indices_plot = [np.argmin(np.abs(nu-x)) for x in nu_plot]
N_plot = nu_plot.size
#Plot parameters
my_fontsize = 18
my_labelsize = 14
my_lw = 1.5
#Axis parameters
L_min, L_max = -1., 1.
rho_min, rho_max = -0.05, 1.2
rho_ticks = np.arange(0., rho.max()+0.01, step=0.2)
rho_labels = [r'${0:g}$'.format(rho) for rho in rho_ticks]
L_cr_min, L_cr_max = 0., 5.
L_cr_ticks = np.arange(L_cr_min, L_cr_max+0.1, step=1)
L_cr_labels = [r'${0:g}$'.format(L_cr) for L_cr in L_cr_ticks]
nu_min, nu_max = 0.05, 0.55
nu_ticks = np.arange(0.1, 0.5+0.01, step=0.1)
nu_labels = [r'${0:g}$'.format(nu) for nu in nu_ticks]
#Create custom colormap for nu
cmap_list = [cm_blue, cm_greenblue, cm_black, cm_lightbrown, cm_brown]
#Colormap
cmap_plot = colors.LinearSegmentedColormap.from_list('Discrete cmap', cmap_list, nu_plot.size)
#Labels
cmap_labels = [r'${:.1f}$'.format(x) for x in nu_plot]
N_labels = len(cmap_labels)
#Norm
norm_bins = np.arange(nu_plot.size)+0.5
norm_bins = np.insert(norm_bins, 0, np.min(norm_bins) - 1.0)
norm_diff = norm_bins[1:] - norm_bins[:-1]
cmap_ticks = norm_bins[:-1] + norm_diff / 2
cmap_norm = colors.BoundaryNorm(norm_bins, N_labels, clip=True)
cmap_fmt = ticker.FuncFormatter(lambda x, pos: cmap_labels[cmap_norm(x)])

#Figure
fig = plt.figure(figsize=(4,6))
gs = GridSpec(3, 1, figure=fig)

ax_01 = fig.add_subplot(gs[1,:])
ax_01.patch.set_facecolor('white')
#Plot
rho_plot = LineCollection([np.column_stack([L_pert_over_L_xz, rho_cr[i]]) for i in indices_plot], cmap=cmap_plot, linewidths=my_lw, linestyles='-')
rho_plot.set_array(np.arange(N_plot))
ax_01.add_collection(rho_plot)
#Axes
for axis in ['top','bottom','left','right']:
	ax_01.spines[axis].set_linewidth(1.5)

ax_01.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_01.set_yticks(rho_ticks)
ax_01.set_yticklabels(rho_labels)
#Labels
ax_01.set_xlabel(r'Size ratio $\epsilon_{xz} \cdot \ell_\mathrm{pert}/\mathcal{L}_{xz}$', fontsize=my_fontsize)
ax_01.set_ylabel(r'Critical mode mixity'+'\n'+r'ratio $\left[K_\mathrm{III}^0/K_\mathrm{I}^0\right]_\mathrm{cr}$', fontsize=my_fontsize)
#Limits
ax_01.set_xlim(L_min, L_max)
ax_01.set_ylim(rho_min, rho_max)
#Colorbar
ax_cbar = fig.add_subplot(gs[0,:])
cbar = fig.colorbar(cm.ScalarMappable(norm=cmap_norm, cmap=cmap_plot), cax=ax_cbar, orientation='horizontal', format=cmap_fmt, ticks=cmap_ticks)
cbar.ax.set_title(r"Poisson's ratio $\nu$", fontsize=my_fontsize)
cbar.ax.tick_params(labelsize=my_labelsize, width=1.5) 
cbar.outline.set_linewidth(1.5)
cbar.set_alpha(1)
cbar.draw_all()

ax_02 = fig.add_subplot(gs[2,:])
ax_02.patch.set_facecolor('white')
#Plot
ax_02.plot(nu, L_cr_over_L_xz, color='black', linewidth=2.)
#Axes
for axis in ['top','bottom','left','right']:
	ax_02.spines[axis].set_linewidth(1.5)

ax_02.tick_params(axis='x', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.tick_params(axis='y', which='major', direction='out', length=4, width=1.5, labelsize=my_labelsize)
ax_02.set_xticks(nu_ticks)
ax_02.set_xticklabels(nu_labels)
ax_02.set_yticks(L_cr_ticks)
ax_02.set_yticklabels(L_cr_labels)
#Labels
ax_02.set_xlabel(r"Poisson's ratio $\nu$", fontsize=my_fontsize)
ax_02.set_ylabel(r'Critical size ratio'+'\n'+r'$\left[\ell_\mathrm{pert}/\mathcal{L}_{xz}\right]_\mathrm{cr}$', fontsize=my_fontsize)
#Limits
ax_02.set_xlim(nu_min, nu_max)
ax_02.set_xlim(nu_min, nu_max)

##Fix position
#Size coefficients
c_y = 0.1
n_y = 0.7
#Interlines
L_x = 0.25
l_x = 0.05
l_y = 0.03
L_y = 0.06
#Graphic sizes
H_x = (0.98-L_x-l_x)
H_y = (0.98-3*L_y-5*l_y)/(1+n_y+c_y)
C_x = H_x
C_y = c_y*H_y
N_x = H_x
N_y = n_y*H_y
#Set position
ax_cbar.set_position([0.01+1*L_x, 0.01+1*H_y+1*N_y+2*L_y+5*l_y, C_x, C_y])
ax_01.set_position([0.01+1*L_x, 0.01+0*H_y+1*N_y+2*L_y+3*l_y, H_x, H_y])
ax_02.set_position([0.01+1*L_x, 0.01+0*H_y+0*N_y+1*L_y+1*l_y, N_x, N_y])
##Save
fig.savefig('figures/EJM_TXZ_CriticalModeMixity.pdf', format='pdf')
plt.close()

#Data for Zenodo
L_pert_over_L_xz_grid, nu_grid = np.meshgrid(L_pert_over_L_xz, nu[indices_plot])
export_header = r"Poisson's ratio $\nu$; Size ratio $\epsilon_{xz} \cdot \ell_\mathrm{pert}/\mathcal{L}_{xz}$; Critical mode mixity ratio $\left[K_\mathrm{III}^0/K_\mathrm{I}^0\right]_\mathrm{cr}$"
export_table = np.column_stack([nu_grid.flatten(), L_pert_over_L_xz_grid.flatten(), rho_cr[indices_plot].flatten()])
np.savetxt("data/data_Figure_8a.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')
export_header = r"Poisson's ratio $\nu$; Critical size ratio $\left[\ell_\mathrm{pert}/\mathcal{L}_{xz}\right]_\mathrm{cr}$"
export_table = np.column_stack([nu, L_cr_over_L_xz])
np.savetxt("data/data_Figure_8b.txt", export_table, delimiter=";", header = export_header, fmt='%.6e')